import json
import requests
import pandas as pd
import sys
if sys.version_info[0] < 3:
    from StringIO import StringIO
else:
    from io import StringIO

__all__ = [
    'RequestObject',
    'DataSet',
    'EnergyworxClient'
]

#Initialize the values we need to connect to the Energyworx API
#Declare the request object
class RequestObject(object):
  uri = "https://cloud.energyworx.com/rest/datapoint/use/aggregations"
  headers = {'accept': 'text/csv','content-type':'application/json'}
    
  def __init__(self, apiKey, tenant, datasets, startDate, endDate):
    self.datasets = datasets
    self.headers["api-key"] = apiKey
    self.headers["x-tenant-domain"] = tenant
    self.params = {"startDate":startDate, "endDate":endDate}
    self.aggregationInterval = "PER_QUARTER_HOUR"
    self.aggregationPeriod = "IN_GIVEN_PERIOD"
    self.path = RequestObject.uri + "/"+self.aggregationInterval+"/"+self.aggregationPeriod
    
  def url(self):
    return self.path
  
  def payload(self):
    return '{"datasets":'+json.dumps([dataset.__dict__ for dataset in self.datasets])+'}'

#Declare the dataset object
class DataSet(object):
  def __init__(self, identifier, dataSourceType, channelType, unitType, consolidationFunction):
      self.identifier=identifier
      self.consolidationFunction = consolidationFunction
      self.channelType=channelType
      self.unitType=unitType
      self.dataSourceType=dataSourceType
      
  def id(self):
      return self.identifier+":"+self.dataSourceType+":"+self.channelType

#This is the main function for getting the data
class EnergyworxClient(object):

  def __init__(self, req):
      self.req=req

  def request_aggregations():
      r = requests.post(req.url(), params=req.params, headers=req.headers, data=req.payload())
      timeseries_df = pd.read_csv(StringIO(r.text), index_col=0, parse_dates=[0], date_parser=lambda x: pd.datetime.strptime(x, '%d-%m-%Y %H:%M'))
      timeseries_df.index.names = ["Timestamp"]
      return timeseries_df

